package com.quiteJee.poi.word.parse;

import com.quiteJee.poi.word.common.Constant;
import com.quiteJee.poi.word.dto.BGDDto;
import com.quiteJee.poi.word.dto.ConfigDto;
import org.apache.poi.POIXMLDocument;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.quiteJeeframework.poi.excel.utils.ExcelUtils;
import org.quiteJeeframework.utils.FileUtil;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ReadWord {
	static String userpath = System.getProperty("user.dir") + File.separator;
	static List<ConfigDto> configList = null;

	static void init() {
		configList = new ArrayList<>();
		Document document = null;
		System.out.println("加载配备文件：" + userpath + "config" + File.separator + "config.xml");

		SAXReader reader = new SAXReader();
		try {
			document = reader.read(userpath + "config" + File.separator + "config.xml");
		} catch (DocumentException e) {
			e.printStackTrace();
		}

		// document = XMLUtil.getDocument(userpath + "config" + File.separator +
		// "config.xml");
		List<Element> list = document.selectNodes("/config/column");
		for (Element el : list) {
			ConfigDto config = new ConfigDto();
			config.setId(el.selectSingleNode("id").getText());
			config.setName(el.selectSingleNode("name").getText());
			config.setExpression(el.selectSingleNode("expression").getText());
			configList.add(config);
		}

	}

	/**
	 * 得到要解析的文件列表
	 * 
	 * @param path
	 *            word文件路径
	 * @return List<String>
	 */
	@SuppressWarnings("unchecked")
	public List<String> getFileList(String path) {
		return FileUtil.getListFiles(path, null, true);

	}

	public static void main(String[] args) {
		init();
		System.out.println("开始解析");
		ReadWord read = new ReadWord();
		String wordsPath = userpath + "word";
		List<String> list = read.getFileList(wordsPath);
		if (list.size() == 0) {
			System.out.println("没有文件需要解析！");
			return;
		}
		List<BGDDto> dtoList = new ArrayList<>();
		for (String file : list) {
			// 判断文件是否为word文档
			if (!file.endsWith(".doc") && !file.endsWith(".docx")) {
				continue;
			}
			// 文档内容
			StringBuffer wordContext = new StringBuffer("");
			// 读取word内容
			try {
				XWPFDocument doc = new XWPFDocument(POIXMLDocument.openPackage(file));

				List<XWPFParagraph> paras = doc.getParagraphs();
				for (XWPFParagraph para : paras) {

					wordContext.append(para.getText());
					wordContext.append("\n");
				}
				// System.out.println(wordContext.toString());

				BGDDto dto = parse(file, wordContext.toString());
				dtoList.add(dto);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		System.out.println("解析完成！");
		System.out.println("开始生成excel");
		// 生成excel
		String model = userpath + "config" + File.separator + "model.xlsx";
		String resultPath = userpath + "excel" + File.separator + "result.xlsx";
		//生成总计
		BGDDto dto  = new BGDDto();
		dto.setTitile("总计：");
		for(BGDDto d:dtoList){
			dto.setAbry(dto.getAbry()+d.getAbry());
			dto.setBgdry(dto.getBgdry()+d.getBgdry());
			dto.setCs(dto.getCs()+d.getCs());
			dto.setFdc(dto.getFdc()+d.getFdc());
			dto.setFdj(dto.getFdj()+d.getFdj());
			dto.setJsry(dto.getJsry()+d.getJsry());
			dto.setTrcl(dto.getTrcl()+d.getTrcl());
			dto.setWwry(dto.getWwry()+d.getWwry());
			dto.setXscd(dto.getXscd()+d.getXscd());
			dto.setZbba(dto.getZbba()+d.getZbba());
			dto.setZbry(dto.getZbry()+d.getZbry());
			dto.setZsry(dto.getZsry()+d.getZsry());
		}
		dtoList.add(dto);
		try {
			ExcelUtils.getInstance().exportObjects2Excel(model, 0, dtoList, null, BGDDto.class, false, resultPath);
		} catch (Exception e) {
			e.printStackTrace();
		}
		System.out.println("生成excel完成,文件位置：" + resultPath);

	}

	private static BGDDto parse(String fileName, String context) {
	String[]	f=fileName.split("\\\\");
	 
		System.out.println("解析：" + fileName);
		BGDDto dto = new BGDDto();
		String fileNameRex = "[\u4E00-\u9FA5]*供电局";// 文件名称解析
		Pattern fr = Pattern.compile(fileNameRex);
		Matcher fm = fr.matcher(fileName);
		if (fm.find()) {
			//dto.setTitile(fm.group(0));// 供电局
		}
		dto.setTitile(f[f.length-1]);
		for (ConfigDto config : configList) {
			String rex = config.getExpression();
			String numRex = "\\d++";
			Pattern r = Pattern.compile(rex);
			Pattern rr = Pattern.compile(numRex);
			Matcher m = r.matcher(context);
			if (m.find()) {
				int value = 0;
				String desc = m.group(0);
				Matcher mm = rr.matcher(desc);
				if (mm.find()) {
					value = Integer.parseInt(mm.group(0));
				}
				switch (config.getId()) {
				case Constant.DATE:
					dto.setDate(m.group(0));
					break;
				case Constant.ZBBA:
					dto.setZbba(value);
					break;
				case Constant.XSCD:
					dto.setXscd(value);
					break;
				case Constant.CS:
					dto.setCs(value);
					break;
				case Constant.BGDRY:
					dto.setBgdry(value);
					break;
				case Constant.ZBRY:
					dto.setZbry(value);
					break;
				case Constant.JSRY:
					dto.setJsry(value);
					break;
				case Constant.ZSRY:
					dto.setZsry(value);
					break;
				case Constant.ABRY:
					dto.setAbry(value);
					break;
				case Constant.WWRY:
					dto.setWwry(value);
					break;
				case Constant.TRCL:
					dto.setTrcl(value);
					break;
				case Constant.FDC:
					dto.setFdc(value);
					break;
				case Constant.FDJ:
					dto.setFdj(value);
					break;

				default:
					break;
				}

			}
		}
		return dto;
	}

}
